/* 
 *
 * Memory Streams - Haran Shivanan (Dec,2002)
 * Portable (mostly) code and data structures for implementing generic
 * memory streams that are allocated on the heap and auto-
 * expand.
 * They are designed to be used like FILE streams.
 * Simple and usefull for string concatanation.
 *
 *
 * 
 * For licensing terms, see the file COPYING in this directory
 * For contact information, see the file AUTHORS in this directory
 *
 */
#include <stdio.h>
#include <string.h>
#include "stream.h"

static int stream_expand(STREAM * stream, size_t new_capacity)
{
	char *tmp = (char *)reallocate(stream->buffer, new_capacity);
	if (!tmp)
		return 0;
	/*if (tmp!=stream->buffer)
	{
		fprintf(stderr,"NOREALLOC:%i",new_capacity);
		fprintf(stderr,tmp);
		fprintf(stderr,stream->buffer);
		exit(-1);
	}*/
	//memcpy(tmp,stream->buffer,stream->size);
	stream->capacity = new_capacity;
	stream->buffer = tmp;
	return 1;
	
	
}

STREAM * stream_create (size_t size)
{
	STREAM *result = (STREAM *)allocate(sizeof(STREAM));
	result->size = 0;
	result->capacity = size;
	result->offset = 0;
	result->buffer = (char *)allocate(size);
	if (!result->buffer)
		return NULL;
	return result;
}
void stream_free(STREAM *stream)
{
	if (stream->buffer)
		deallocate(stream->buffer);
	deallocate(stream);
}


size_t stream_write(STREAM *stream,char *data,size_t len)
{
	int final_size;
	
	/*check if the internal buffer has enough space for the
	 * incomming data */
	final_size  = stream->offset+len;
	if (final_size > stream->capacity)
	{
		/*if we need to expand to more than twice the current size
		 * ,just expand by the required amount*/
		if (final_size > 2*stream->capacity)
			stream_expand(stream,final_size);
		else
		/* otherwise, double the capacity for every expansion */
			stream_expand(stream,2*stream->capacity);
	}
	memcpy(stream->buffer+stream->offset,data,len);
	stream->offset += len;
	if (final_size > stream->size)
		stream->size = final_size;
	return len;
}

/* same as stream_write but without the overhead
 * of memcpy(...)
 */
int stream_add_char(STREAM *stream,char ch)
{
	int final_size;
	
	/*check if the internal buffer has enough space for the
	 * incomming data */
	final_size  = stream->offset+1;
	if (final_size > stream->capacity)
	{
		/*if we need to expand to more than twice the current size
		 * ,just expand by the required amount*/
		if (final_size > 2*stream->capacity)
			stream_expand(stream,final_size);
		else
		/* otherwise, double the capacity for every expansion */
			stream_expand(stream,2*stream->capacity);
	}
	stream->buffer[stream->offset]=ch;
	stream->offset++;
	if (final_size > stream->size)
		stream->size = final_size;
	return 1;	
}

char * stream_data(STREAM *stream)
{
	return stream->buffer;
}



#ifdef DO_TEST
int main(int argc, char **argv)
{
	STREAM *stream;
	stream = stream_create(10);
	while (argc-->0)
	{
		//printf("%s:%i\n",*argv,strlen(*argv));
		//fflush(stdout);
		stream_write(stream,*argv,strlen(*argv));
		stream_write(stream,"\n",1);
		argv++;
	}
	//stream_write(stream,"\0",1);
	stream->buffer[stream->size]=0;
	printf(stream->buffer);
}
#endif //DO_TEST
